local BG = LibStub("AceAddon-3.0"):GetAddon("Battleground")
local module = BG:NewModule("ArathiBasin", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Battleground")

local landmarks = BG.landmarks
local namedButtons = BG.namedButtons

module.captureTimer = 62

module.buttons = {
	left = {
		L["Stables"],
		L["Gold Mine"],
		L["Blacksmith"],
		L["Lumber Mill"],
		L["Farm"]
	},
	bottom = {
		L["Final Score: %d/%d"],
		L["Bases to win: %d"],
		
	},
	top = {
		L["Battleground Start"],
	},
}

-- damn exceptions in chat
local pois = {
	[L["mine"]] = L["Gold Mine"],
	[L["lumber mill"]] = L["Lumber Mill"],
	[L["blacksmith"]] = L["Blacksmith"],
	[L["farm"]] = L["Farm"],
	[L["stables"]] = L["Stables"],
}


local ppbps = {  -- points / bases / sec
	[0] = 0,
	[1] = 0.8333,
	[2] = 1.1111,
	[3] = 1.6667,
	[4] = 3.3333,
	[5] = 30, 
}

local horde = {}
local alliance = {}
local playerFaction 

function module:OnEnable()
	playerFaction = playerFaction or UnitFactionGroup("player")
	wipe(horde)
	wipe(alliance)
	self:RegisterEvent("UPDATE_WORLD_STATES")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE", "ParsePerson")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE", "ParsePerson")
end

local function getState(id)
	local text = select(3, GetWorldStateUIInfo(id))
	if not text then return end
	local bases, points, maxpoints = string.match(text, L["Bases: ([0-9]+)  Resources: ([0-9]+)/([0-9]+)"])
	points = tonumber(points)
	bases = tonumber(bases)
	maxpoints = tonumber(maxpoints)
	if not bases or not points or not maxpoints then return end
	return bases, points, maxpoints
end

local function getFinalScore(time, bases)
	if bases == 0 then return 0 end
	return floor((time * ppbps[bases] + 0.5) / 10) * 10
end

function module:ParsePerson(event, msg)
	local name, poi = msg:match(L["(.+) claims the ([^!]+)"])
	if not name then
		name, poi = msg:match(L["(.+) has assaulted the ([^!]+)"])
	end
	if name then
		name = name:trim()
		poi = poi:trim()
		poi = pois[poi:lower()]
		for k, v in pairs(namedButtons) do
			if k == poi then
				v.text:SetFormattedText("[%s] %s", name, k)
				break
			end
		end
		return
	end
	poi = msg:match(L["has taken the ([^!]+)"])
	if not poi then
		poi = msg:match(L["has defended the ([^!]+)"])
	end
	if poi then
		poi = poi:trim()
		poi = pois[poi:lower()]
		for k, v in pairs(namedButtons) do
			if k == poi then
				v.text:SetText(k)
				break
			end
		end
		return
	end
end

function module:UPDATE_WORLD_STATES()
	local aBases, aPoints, aMPoints = getState(1)
	local hBases, hPoints, hMPoints = getState(2)
	if not aBases or not hBases or (aPoints == 0 and hPoints == 0) then return end
	
	alliance.bases = aBases
	alliance.points = aPoints
	alliance.max_points = aMPoints
	alliance.left = aMPoints - aPoints
	alliance.time = alliance.left / ppbps[aBases]

	horde.bases = hBases
	horde.points = hPoints
	horde.max_points = hMPoints
	horde.left = hMPoints - hPoints
	horde.time = horde.left / ppbps[hBases]

	local lowest = min(horde.time, alliance.time)
	alliance.final = alliance.points + getFinalScore(lowest, alliance.bases)
	horde.final = horde.points + getFinalScore(lowest, horde.bases)

	local ti = 46
	local color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
	if horde.final > alliance.final then
		ti = 48
		color = ChatTypeInfo["BG_SYSTEM_HORDE"]
	end
	namedButtons[L["Final Score: %d/%d"]].icon:SetTexCoord(landmarks[ti][3], landmarks[ti][4], landmarks[ti][5], landmarks[ti][6])
	namedButtons[L["Final Score: %d/%d"]].text:SetText(L["Final Score: %d/%d"]:format(alliance.final, horde.final))
	namedButtons[L["Final Score: %d/%d"]].text:SetTextColor( color.r, color.g, color.b)
	namedButtons[L["Final Score: %d/%d"]].expire = time() + lowest
	namedButtons[L["Final Score: %d/%d"]]:SetScript("OnUpdate", BG.OnUpdateCountDown)
	namedButtons[L["Final Score: %d/%d"]]:Show()
	if playerFaction == FACTION_HORDE then
		friend = horde
		enemy = alliance
	else
		friend = alliance
		enemy = horde
	end

	local basestowin = 5
	for i = 1, 5 do
		local lowest = min(enemy.left / ppbps[5-i], friend.left / ppbps[i])
		local enemyfinal = enemy.points + getFinalScore(lowest, 5-i)
		local friendfinal = friend.points + getFinalScore(lowest, i)
		if friendfinal >= friend.max_points and enemyfinal < enemy.max_points then
			basestowin = i 
			break
		end
	end
	namedButtons[L["Bases to win: %d"]].text:SetText(L["Bases to win: %d"]:format(basestowin))
	namedButtons[L["Bases to win: %d"]]:Show()
end