local BG = LibStub("AceAddon-3.0"):GetAddon("Battleground")
local module = BG:NewModule("NetherstormArena", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Battleground")

local landmarks = BG.landmarks
local namedButtons = BG.namedButtons

module.captureTimer = 62
module.stateRows = 3

module.buttons = {
	left = {
		L["Mage Tower"],
		L["Draenei Ruins"],
		L["Fel Reaver Ruins"],
		L["Blood Elf Tower"],
	},
	bottom = {
		L["Final Score: %d/%d"],
		L["Bases to win: %d"],
		
	},
	top = {
		L["Battleground Start"],
	},
}

local ppbps = {  -- points / bases / sec
	[0] = 0,
	[1] = 0.5,
	[2] = 1,
	[3] = 2.5,
	[4] = 5
}

local horde = {}
local alliance = {}
local playerFaction 

function module:OnEnable()
	playerFaction = playerFaction or UnitFactionGroup("player")
	wipe(horde)
	wipe(alliance)
	self:RegisterEvent("UPDATE_WORLD_STATES")
end

local function getState(id)
	local text = select(3, GetWorldStateUIInfo(id))
	if not text then return end
	local bases, points, maxpoints = string.match(text, L["Bases: ([0-9]+)  Victory Points: ([0-9]+)/([0-9]+)"])
	points = tonumber(points)
	bases = tonumber(bases)
	maxpoints = tonumber(maxpoints)
	if not bases or not points or not maxpoints then return end
	return bases, points, maxpoints
end

local function getFinalScore(time, bases)
	if bases == 0 then return 0 end
	return floor(time * ppbps[bases] + 0.5)
end

function module:UPDATE_WORLD_STATES()
	local aBases, aPoints, aMPoints = getState(2)
	local hBases, hPoints, hMPoints = getState(3)
	if not aBases or not hBases or (aPoints == 0 and hPoints == 0) then return end
	
	alliance.bases = aBases
	alliance.points = aPoints
	alliance.max_points = aMPoints
	alliance.left = aMPoints - aPoints
	alliance.time = alliance.left / ppbps[aBases]

	horde.bases = hBases
	horde.points = hPoints
	horde.max_points = hMPoints
	horde.left = hMPoints - hPoints
	horde.time = horde.left / ppbps[hBases]

	local lowest = min(horde.time, alliance.time)
	alliance.final = alliance.points + getFinalScore(lowest, alliance.bases)
	horde.final = horde.points + getFinalScore(lowest, horde.bases)

	local ti = 46
	local color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
	if horde.final > alliance.final then
		ti = 48
		color = ChatTypeInfo["BG_SYSTEM_HORDE"]
	end
	namedButtons[L["Final Score: %d/%d"]].icon:SetTexCoord(landmarks[ti][3], landmarks[ti][4], landmarks[ti][5], landmarks[ti][6])
	namedButtons[L["Final Score: %d/%d"]].text:SetText(L["Final Score: %d/%d"]:format(alliance.final, horde.final))
	namedButtons[L["Final Score: %d/%d"]].text:SetTextColor( color.r, color.g, color.b)
	namedButtons[L["Final Score: %d/%d"]].expire = time() + lowest
	namedButtons[L["Final Score: %d/%d"]]:SetScript("OnUpdate", BG.OnUpdateCountDown)
	namedButtons[L["Final Score: %d/%d"]]:Show()
	if playerFaction == FACTION_HORDE then
		friend = horde
		enemy = alliance
	else
		friend = alliance
		enemy = horde
	end

	local basestowin = 4
	for i = 1, 4 do
		local lowest = min(enemy.left / ppbps[4-i], friend.left / ppbps[i])
		local enemyfinal = enemy.points + getFinalScore(lowest, 4-i)
		local friendfinal = friend.points + getFinalScore(lowest, i)
		if friendfinal >= friend.max_points and enemyfinal < enemy.max_points then
			basestowin = i 
			break
		end
	end
	namedButtons[L["Bases to win: %d"]].text:SetText(L["Bases to win: %d"]:format(basestowin))
	namedButtons[L["Bases to win: %d"]]:Show()
end